@@ -1,27 +1,58 @@
 /*
- *      This file is part of GPaste.
+ * This file is part of GPaste.
  *
- *      Copyright 2011-2015 Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
- *
- *      GPaste is free software: you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation, either version 3 of the License, or
- *      (at your option) any later version.
- *
- *      GPaste is distributed in the hope that it will be useful,
- *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
- *
- *      You should have received a copy of the GNU General Public License
- *      along with GPaste.  If not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2010-2018, Marc-Antoine Perennou <Marc-Antoine@Perennou.com>
  */
 
+#include <gpaste-gtk3/gpaste-gtk-macros.h>
+
 #include <gpaste-bus.h>
 #include <gpaste-daemon.h>
 #include <gpaste-search-provider.h>
 
-static GApplication *_app;
+#ifdef G_OS_UNIX
+#  include <glib-unix.h>
+#endif
+
+static void
+reexec (GPasteDaemon *g_paste_daemon G_GNUC_UNUSED,
+        gpointer      user_data)
+{
+    GApplication *app = user_data;
+
+    g_application_quit (app);
+
+    execl (PKGLIBEXECDIR "/gpaste-daemon", "gpaste-daemon", NULL);
+}
+
+#ifdef G_OS_UNIX
+static gboolean
+signal_handler (gpointer user_data)
+{
+    GApplication *app = user_data;
+
+    g_print ("%s\n", _("Stop signal received, exiting"));
+    g_application_quit (app);
+
+    return G_SOURCE_REMOVE;
+}
+
+typedef struct
+{
+    GPasteDaemon *daemon;
+    GApplication *app;
+} Usr1Data;
+
+static gboolean
+usr1_handler (gpointer user_data)
+{
+    Usr1Data *data = user_data;
+
+    reexec (data->daemon, data->app);
+
+    return G_SOURCE_REMOVE;
+}
+#endif
 
 enum
 {
@@ -31,13 +62,6 @@ enum
     C_LAST_SIGNAL
 };
 
-static void
-signal_handler (gint32 signum)
-{
-    g_print ("Signal %" G_GINT32_FORMAT " received, exiting\n", signum);
-    g_application_quit (_app);
-}
-
 G_GNUC_NORETURN static void
 on_name_lost (GPasteBus *bus G_GNUC_UNUSED,
               gpointer   user_data)
@@ -46,6 +70,7 @@ on_name_lost (GPasteBus *bus G_GNUC_UNUSED,
 
     fprintf (stderr, "%s\n", _("Could not acquire DBus name."));
     g_application_quit (app);
+
     exit (EXIT_FAILURE);
 }
 
@@ -87,17 +112,7 @@ on_bus_acquired (GPasteBus *bus,
 
     register_bus_object (bus, G_PASTE_BUS_OBJECT (data->daemon), data->gapp);
 
-    g_idle_add (register_search_provider, user_data);
-}
-
-static void
-reexec (GPasteDaemon *g_paste_daemon G_GNUC_UNUSED,
-        gpointer      user_data)
-{
-    GApplication *app = user_data;
-
-    g_application_quit (app);
-    execl (PKGLIBEXECDIR "/gpaste-daemon", "gpaste-daemon", NULL);
+    g_source_set_name_by_id (g_idle_add (register_search_provider, user_data), "[GPaste] register_search_provider");
 }
 
 gint
@@ -106,7 +121,7 @@ main (gint argc, gchar *argv[])
     /* FIXME: remove this once gtk supports clipboard correctly on wayland */
     gdk_set_allowed_backends ("x11");
 
-    G_PASTE_INIT_APPLICATION ("Daemon");
+    G_PASTE_GTK_INIT_APPLICATION ("Daemon");
 
     /* Keep the gapplication around */
     g_application_hold (gapp);
@@ -116,11 +131,11 @@ main (gint argc, gchar *argv[])
     g_autoptr (GPasteBusObject) search_provider = NULL;
 
     data->search_provider = &search_provider;
-    _app = data->gapp = gapp;
+    data->gapp = gapp;
 
     g_autoptr (GPasteBus) bus = data->bus = g_paste_bus_new (on_bus_acquired, data);
 
-    gulong c_signals[C_LAST_SIGNAL] = {
+    guint64 c_signals[C_LAST_SIGNAL] = {
         [C_NAME_LOST] = g_signal_connect (bus,
                                           "name-lost",
                                           G_CALLBACK (on_name_lost),
@@ -131,11 +146,19 @@ main (gint argc, gchar *argv[])
                                                gapp)
     };
 
-    signal (SIGTERM, &signal_handler);
-    signal (SIGINT, &signal_handler);
+#ifdef G_OS_UNIX
+    g_source_set_name_by_id (g_unix_signal_add (SIGTERM, signal_handler, app), "[GPaste] SIGTERM listener");
+    g_source_set_name_by_id (g_unix_signal_add (SIGINT,  signal_handler, app), "[GPaste] SIGINT listener");
+
+    Usr1Data usr1_data = { g_paste_daemon, gapp };
+
+    g_source_set_name_by_id (g_unix_signal_add (SIGUSR1, usr1_handler, &usr1_data), "[GPaste] SIGUSR1 listener");
+#endif
 
     g_paste_bus_own_name (bus);
 
+    g_paste_util_write_pid_file ("Daemon");
+
     gint64 exit_status = g_application_run (gapp, argc, argv);
 
     g_signal_handler_disconnect (bus, c_signals[C_NAME_LOST]);
